function [xmin,fmin,gmin,fcount] = FSEDA(f,U,L)
% *************************************************************************
% FSEDA = Fuzzy Sampling Estimation of Distribution Algorithm 
% The FSEDA Code is created on March 27, 2019 by A. Hedar & A.A. Allam
% The code is modified on Sep. 20, 2020.
% The FSEDA method are submitted to Appl. Sci. Journal, MDPI
% The user is refereed to the paper to alter the code for an application 
% *************************************************************************
% Inputs:
%  f    = Objective function
%  U    = n-Dimension Vector of upper limits of the objective variables
%  L    = n-Dimension Vector of lower limits of the objective variables
%
% Outputs:
%  xmin   = Best point obtained by EDA
%  fmin   = the best function value obtained by FSEDA with sampling 
%  gmin   = the exact best function value obtained by FSEDA
%  fcount = Number of function evaluations                      
% ************************************************************************* 

% Initial Parameters
global noise nSample
noise = 1;
n  = length(U);      % No. of variables
R  = 100;  
Rcorr = 100;        % Initial Population size
Rsamp = R - 10;
s  = ceil(0.15*R)  ;  % Selected individuals         
c  = 5;
max_fc = 5000*n; 
fcount = 0;
Nsamp = 10;
N_max = 10000;
NsampUp = Nsamp;
nSample = 5;
local_search = 0;

% Creating the initial population
P  = DivInSet(U,L,R);
fP = zeros(1,R);
for i=1:R 
    [fP(1,i)] = Sampling(f,P(:,i),Nsamp);
end
fcount = fcount + Nsamp * R;
xmin = [];
fmin = Inf;
% ************************************************************************* 
% Main Iteration
terminate = 0;
n_Itr = 1;
while terminate == 0 
    if Nsamp > N_max
        Nsamp = N_max;
    end
    fNP = zeros(1,Rsamp);
    NewPop1 = [];
    [fPs,is] = sort(fP); P_tmp = P(:,is); P = P_tmp; fP = fPs;    
    [xmin,fmin] = updateBest(f,P(:,1),xmin,fmin); 
    fcount = fcount + 25;
    Pcorr = P(:,1:Rcorr);
    Psel = P(:,1:s);
    [inW,inS] = WeaklyDependent(Pcorr,n,0.3);
    W = Psel(inW,:);
    tf = isempty(W);
    if tf == 0
        W = uniGaussian(W,Rsamp);
        NewPop1 = [NewPop1 ; W];
    end
    S = Psel(inS,:);
    tf = isempty(S);
    if tf == 0
        S = subspace(S,Rsamp,c);
        NewPop1 = [NewPop1 ; S];
    end 
    index = [inW;inS];
    [~,is] = sort(index);
    NewPop = NewPop1(is,:);
    for i=1 :Rsamp 
        fNP(1,i) = Sampling(f,NewPop(:,i),Nsamp);
        fcount = fcount + Nsamp;        
    end
    P = [P(:,1:10) NewPop];
    fP = [fP(1,1:10) fNP];
    [fPs,is] = sort(fP); P_tmp = P(:,is); P = P_tmp; fP = fPs;
    if fP(1,1) < fmin        
        if local_search == 1             
            [P(:,1),fP(1),xmin,fmin,fCOUNT] = ...
                fminLocalSearch(f,P(:,1),fP(1),xmin,fmin,n);
            fcount = fcount + fCOUNT;
        end        
        [xmin,fmin] = updateBest(f,P(:,1),xmin,fmin);
        fcount = fcount + 25;        
    end
    % Termination conditions
    n_Itr = n_Itr+1;
    if ( fcount > max_fc)
        terminate = 1;  
    end
    NsampUp = NsampUp * 1.025;
    Nsamp = round(NsampUp);    
end

noise = 0;
gmin = feval(f,xmin);
end